#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

//////////////////////////////////////////////////////////////////////////////////////////////////
// uncomment this to get some debug printout
//#define DEBUG 

//////////////////////////////////////////////////////////////////////////////////////////////////
int fdin[5];                    // keeps the filedescriptors to each stripe file
char stripebuffer[5][65536];    // stores 64k buffers for each stripe file
int nreadchunk=0;               // counts the number of chunks we have read starting from 1
const char* filename=0;         // stores the basename of the stripe files without *-0,*-1 ... *-4
//////////////////////////////////////////////////////////////////////////////////////////////////




int main(int argc, char* argv[]) {
  filename = argv[1];

  //////////////////////////////////////////////////////////////////////////////////////////////////
  // check that the user specified a filename to process
  if (!filename) { fprintf(stderr, "usage: %s <filename>\n", argv[0]); exit(-1);}

  //////////////////////////////////////////////////////////////////////////////////////////////////
  // open all the stripes, this maps stripe file 0 to fdin[0] .... 4 to fdin[4]
  if (!open_stripes()) {
    exit(-1);
  }

  //////////////////////////////////////////////////////////////////////////////////////////////////
  for (;;) {
    //////////////////////////////////////////////////////////////////////////////////////////////////
    // this loads the next 64k block from stripe 0 into stripebuffer[0] .... 4 into stripebuffer[4]
    if (!load_stripes()) {
      // if we can't load anymore a stripe, terminate the file loop
      break;
    }    
    // count the chunk number we have read per stripe
    nreadchunk++;
#ifdef DEBUG
    printf("|=> loaded chunk%d\n",nreadchunk);
#endif
    //////////////////////////////////////////////////////////////////////////////////////////////////
    // compute parities here and print faulty bytes!
    {

    }
    //////////////////////////////////////////////////////////////////////////////////////////////////
  } 

  fprintf(stderr,"info: analyzed %d stripe chunks\n",nreadchunk);
  exit(0);
} // end of main

//////////////////////////////////////////////////////////////////////////////////////////////////
// convenience routines
//////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////
// routine to open all stripe files and attach to filedescriptors fdin[0-4]
int open_stripes() {
  int nstripe;
  for (nstripe=0; nstripe < 5; nstripe++) {
    char stripefilename[4096];
    sprintf(stripefilename,"%s-%d",filename,nstripe);
    // open stripe output files
    if ( ((fdin[nstripe] = open(stripefilename, O_RDONLY)) <= 0 ) ) {
      fprintf(stderr,"error: cannot open %s\n",stripefilename);
      return 0;
    }
#ifdef DEBUG
    printf("|=> opened stripefile %s\n",stripefilename);
#endif
  }
  return 1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
// routine to load the next 64k into the 5 stripe buffers until there is nothing left
int load_stripes() {
  int nstripe;
  // load all 5 chunks into the memory buffers
  for (nstripe=0; nstripe < 5 ; nstripe++) {
    if ((read(fdin[nstripe],stripebuffer[nstripe], 64*1024))!= (64*1024)) {
      // we terminate the do loop, when we don't read anymore a full stripe
      return 0;
    }
  }
  return 1;
}
